require 'jruby_mahout'
require 'tmpdir'

class RecommenderCore::Handler

  def self.similar_users(params)
    recommender = init_recommender(params[:hash])
    recommender.similar_users(params[:id], params[:nb_wanted], params[:rescorer])
  end

  def self.similar_items(params)
    recommender = init_recommender(params[:hash])
    sim_items = recommender.similar_items(params[:id], params[:nb_wanted], params[:rescorer])
    sim_items.map!{ |item| {id: item.getItemID, score: item.getValue} }
  end

  def self.recommend_items(params)
    recommender = init_recommender(params[:hash])
    recommender.recommend(params[:id], params[:nb_wanted], params[:rescorer])
  end

  def self.evaluate(params)
    recommender = init_recommender(params[:hash])
    recommender.evaluate(params[:training_prcentage], params[:eval_prcentage])
  end


  def self.estimate_preference(params)
    recommender = init_recommender(params[:hash])
    recommender.estimate_preference(params[:user_id], params[:item_id])
  end

  private #################################

  def self.tell_to_do()
  end

  def self.respond_to()
  end

  def self.init_recommender(params)
    recommender = JrubyMahout::Recommender.new(params[:similarity_metric], params[:neighborhood], params[:recommender_algo], true)
    file = data_to_csv(params[:engine_data])
    recommender.data_model = JrubyMahout::DataModel.new('file', {file_path: file.path}).data_model
    recommender
  end

  def self.data_to_csv(engine_data)
    file = File.new(return_tmp_path, "w+")
    return nil unless file
    engine_data.each do |row|
      file.puts "#{row[:customer_id]},#{row[:item_id]},#{row[:preference]}"
    end
    file.rewind
    file
  end

  def self.free_file(file)
    File.delete(file)
  end

  def self.return_tmp_path
    File.join(Dir.tmpdir, Time.now.to_s + ".csv")
  end
end
